#------------------------------------------------------------------------------
# Copyright 2008-2011 Istituto Nazionale di Fisica Nucleare (INFN)
#
# Licensed under the EUPL, Version 1.1 only (the "Licence").
# You may not use this work except in compliance with the Licence.
# You may obtain a copy of the Licence at:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Unless required by applicable law or agreed to in
# writing, software distributed under the Licence is
# distributed on an "AS IS" basis,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied.
# See the Licence for the specific language governing
# permissions and limitations under the Licence.
#------------------------------------------------------------------------------
"""
Part of WNoDeS framework.
Interface between WNoDeS_bait process and the LSF batch system
"""

#import time
#from wnodes.utils import utils
#import os
import sys
import threading
import commands

try:
    from xml.etree import ElementTree
except ImportError:
    try:
        from elementtree import ElementTree  # pylint: disable-msg=F0401
    except ImportError:
        sys.exit("package ElementTree is missing, exiting.")


class pbsCmds:
    """ Set of lsf batch system commands"""
    def __init__(self, expiration=15):

        self.expiration = expiration

    def bjobs(self, batchjobid=''):

        try:
            batchjobid = batchjobid.split('.')[0]
        except:
            batchjobid = batchjobid

        cmdLine = ('qstat -x %s' % batchjobid)
        bjobsCmdOutput = commands.getstatusoutput(cmdLine)

        if bjobsCmdOutput[0] == 0:

            JOB_INFO = ElementTree.fromstring(bjobsCmdOutput[1])
            for i in JOB_INFO.getiterator():
                if i.tag == 'job_state':
                    JOB_STATUS = i.text

            if JOB_STATUS == 'Q':
                BATCH_JOB_STATUS = 'PEND'
            elif JOB_STATUS == 'R':
                BATCH_JOB_STATUS = 'RUN'
            elif 'UNKNOWN' in bjobsCmdOutput[1].upper():
                BATCH_JOB_STATUS = 'DONE'
            else:
                BATCH_JOB_STATUS = 'WAITING'
        else:

            # handle the case where the job is finished
            # or the batch system does not respond to the query because busy
            if 'UNKNOWN' in bjobsCmdOutput[1].upper():
                BATCH_JOB_STATUS = 'DONE'
            else:
                BATCH_JOB_STATUS = 'WAITING'

        return [0, ' '.join(['00', '11', str(BATCH_JOB_STATUS)])]

    def breserve(self, *arguments):
    # def breserve(self, action, BATCH_JOB_ID, HOSTNAME, RESERVATION_ID):

        action = arguments[0]

        if action == 'add':

            BATCH_JOB_ID = arguments[1]
            HOSTNAME = arguments[2]

            cmdLine = ('qalter -l nodes=1:%s %s') % (HOSTNAME, BATCH_JOB_ID)
            breserveCmdOutput = commands.getstatusoutput(cmdLine)

            if breserveCmdOutput[0] == 0:
                #cmdLine = ('qmgr -c "set node %s properties += %s"')
                # % (HOSTNAME, RESERVATION_ID)
                #bmodCmdOutput = commands.getstatusoutput(cmdLine)

                #breserveCmdOutput = bmodCmdOutput
                pass

        elif action == 'del':

            BATCH_JOB_ID = arguments[1]
            cmdLine = ('qalter -l nodes=1:bait %s') % (BATCH_JOB_ID)
            breserveCmdOutput = commands.getstatusoutput(cmdLine)

        return breserveCmdOutput

    def bmod(self, bmodOption):

        return [0, 'bmod fatto nel passo precedente']

    def badmin(self, badminOption, host):

        if 'hopen' in badminOption:
            cmdLine = ('pbsnodes  -c %s') % (host)
        elif 'hclose' in badminOption:
            cmdLine = ('pbsnodes  -o %s') % (host)

        badminCmdOutput = commands.getstatusoutput(cmdLine)

        return badminCmdOutput

    def brun(self, HOSTNAME, LOCAL_DOMAIN, BATCH_JOBID):

        cmdLine = ('qrun -H %s.%s %s') % (HOSTNAME, LOCAL_DOMAIN, BATCH_JOBID)
        brunCmdOutput = commands.getstatusoutput(cmdLine)

        return brunCmdOutput

    def bhost(self, HOSTNAME, LOCAL_DOMAIN):

        cmdLine = ('pbsnodes -x %s.%s' % (HOSTNAME, LOCAL_DOMAIN))
        bhostCmdOutput = commands.getstatusoutput(cmdLine)

        if bhostCmdOutput[0] == 0:
            try:
                HOST_INFO = ElementTree.fromstring(bhostCmdOutput[1])

                for i in HOST_INFO.getiterator():
                    if i.tag == 'state':
                        HOST_STATUS = i.text.upper()
            except:
                HOST_STATUS = 'UNKNOWN'
        else:
            if 'UNKNOWN' in bhostCmdOutput[1].upper():
                HOST_STATUS = 'UNKNOWN'
            else:
                HOST_STATUS = 'WAITING'

    def bsub(self, USER, QSUB_OPTION):

        cmdLine = ('su -l %s -c "qsub %s' % (USER, QSUB_OPTION))
        qsubCmdOutput = commands.getstatusoutput(cmdLine)

        return qsubCmdOutput

    def bkill(self, batchJobID, user=""):
        """ Wrapper of the bkill cmd.
        """

        if user is None:
            cmdLine = "qdel %s" % batchJobID
        else:
            cmdLine = 'su -l %s -c "qdel %s"' % (user, batchJobID)

        print "Executing bkill:"
        print '\t' + cmdLine
        bjobsCmdOutput = commands.getstatusoutput(cmdLine)

        return bjobsCmdOutput


class lsfCmds:
    """ Set of lsf batch system commands"""
    LOCK_BJOBS = threading.Lock()
    LOCK_BHOSTS = threading.Lock()

    def __init__(self, lsfProfile,  expiration=15):

        self.lsfProfile = lsfProfile
        self.expiration = expiration

        self.expirationPeriod = [0, 0]
        self.bjobsCmdOutput = []
        self.jobsInfo = {}
        self.hostList = {}

        #bait_id = raw_input("Type the 2 final numbers of active bait: ")
        #self.bait = "vwn-05%s-bait" %bait_id
        self.bait = ""

    def bjobs(self, lsfjobid="", user=""):
        """ Wrapper of bjobs cmd.
        Return a list with the value of bjobs output """

        # bjobs cache initialization

        cmdLine = ('source %s; bjobs -w') % self.lsfProfile
        if user != "":
            cmdLine += " -u %s" % user
        if lsfjobid != "":
            cmdLine += " %s| grep %s" % (lsfjobid, lsfjobid)
        #print "Executing: %s" %cmdLine
        bjobsCmdOutput = commands.getstatusoutput(cmdLine)
        #print "bjobs output: %s" %str(bjobsCmdOutput)
        return  bjobsCmdOutput

    def bmod(self, bmodOption):

        cmdLine = ('source %s; bmod %s') % (self.lsfProfile, bmodOption)

        bmodCmdOutput = commands.getstatusoutput(cmdLine)

        return bmodCmdOutput

    def breserve(self, action, breserveOption):

        if action == 'add':

            cmdLine = ('source %s; brsvadd %s') % (self.lsfProfile,
                                                   breserveOption)
            breserveCmdOutput = commands.getstatusoutput(cmdLine)

        elif action == 'del':

            cmdLine = ('source %s; brsvdel %s') % (self.lsfProfile,
                                                   breserveOption)
            breserveCmdOutput = commands.getstatusoutput(cmdLine)

        return breserveCmdOutput

    def brun(self, hostname, localdomain, jobid):

        cmdLine = ('source %s; brun -m %s %s') % (self.lsfProfile,
                                                  hostname, jobid)
        brunCmdOutput = commands.getstatusoutput(cmdLine)

        return brunCmdOutput

    def badmin(self, badminOption, host):

        cmdLine = ('source %s; ' % self.lsfProfile +
                   'badmin %s ' % badminOption +
                   '-C "WNoDeS admin action" %s' % host)
        badminCmdOutput = commands.getstatusoutput(cmdLine)

        return badminCmdOutput

    def bsub(self, wrapperfile, user=""):
        """ Wrapper of bsub cmd.
        Return a list with the value of bjobs output """

        reserved_bait = ""
        if self.bait != "":
            reserved_bait = "-m %s" % self.bait

        usercmd = ""
        if user != "":
            usercmd = "su -l %s -c" % user

        cmdLine = ('source %s; %s "bsub %s < %s"') % (self.lsfProfile,
                                                      usercmd,
                                                      reserved_bait,
                                                      wrapperfile)

        print "Executing bsub:"
        print '\t' + cmdLine
        bjobsCmdOutput = commands.getstatusoutput(cmdLine)

        return  bjobsCmdOutput

    def bkill(self, batchJobID, user=""):
        """ Wrapper of bsub cmd.
        Return a list with the value of bjobs output """

        cmdLine = 'source %s;' % self.lsfProfile
        if user != "":
            cmdLine += " su -l %s -c" % user
        cmdLine += ' "bkill %s"' % batchJobID

        print "Executing bkill:"
        print '\t' + cmdLine
        bjobsCmdOutput = commands.getstatusoutput(cmdLine)

        return  bjobsCmdOutput
