#------------------------------------------------------------------------------
# Copyright 2008-2011 Istituto Nazionale di Fisica Nucleare (INFN)
#
# Licensed under the EUPL, Version 1.1 only (the "Licence").
# You may not use this work except in compliance with the Licence.
# You may obtain a copy of the Licence at:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Unless required by applicable law or agreed to in
# writing, software distributed under the Licence is
# distributed on an "AS IS" basis,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied.
# See the Licence for the specific language governing
# permissions and limitations under the Licence.
#------------------------------------------------------------------------------
"""
Part of WNoDeS framework.
Interface between WNoDeS processes an the batch system PBS
"""

#import time
from wnodes.utils import command
from wnodes.utils import batch_system

#import os
import sys
import threading
import commands

try:
    from xml.etree import ElementTree
except ImportError:
    try:
        from elementtree import ElementTree  # pylint: disable-msg=F0401
    except ImportError:
        sys.exit("package ElementTree is missing, exiting.")

class Pbsnodes(command.Command):
    def __init__(self, option, hostname):

        self.__hostname__ = hostname
        self.__option__ = option
        self.type_node = {'hopen':'-c',
            'hclose':'-o',
            '':'-x'}

    def _get_command(self):
        cmd = 'pbsnodes %s %s' % (self.type_node[self.__option__],
            self.__hostname__)
        #print cmd
        return cmd

    def get_fake_output(self):
        fake_output = {'0':{'':''},
            '1':{'Unknown':'pbsnodes'}
            }
        return fake_output

class Qalter(command.Command):
    def __init__(self, action, id, hostname = ''):

        self.__hostname__ = hostname
        self.__jobid__ = id
        self.__action__ = action

    def _get_command(self):
        cmd = 'qalter -l nodes=1:'
        if self.__action__ == 'add':
            cmd += '%s %s' % (self.__hostname__, self.__jobid__)
        else:
            cmd += 'bait %s' % self.__jobid__

        #print cmd
        return cmd

    def get_fake_output(self):
        fake_output = ''
        return [0, fake_output]

class Qdel(command.Command):
    def __init__(self, jobid, user = ''):

        self.__jobid__ = jobid
        self.__user__ = user

    def _get_command(self):
        if self.__user__ == '':
            cmd = 'qdel %s' % self.__jobid__
        else:
            cmd = 'su -l %s -c "qdel %s"' % (self.__user__, self.__jobid__)
        #print cmd
        return cmd

    def get_fake_output(self):
        return ['0', '']

class Qrun(command.Command):
    def __init__(self, hostname, jobid):

        self.__hostname__ = hostname
        self.__jobid__ = jobid

    def _get_command(self):
        cmd = 'qrun -H %s %s' % (self.__hostname__, self.__jobid__)
        #print cmd
        return cmd

    def get_fake_output(self):
        fake_output = ''
        return [0, fake_output]

class Qstat(command.Command):
    def __init__(self, jobid = '', user = ''):
        
        self.__user__ = user
        self.__jobid__ = jobid

    def _get_command(self):
        cmd = 'qstat -x %s' % self.__jobid__
        #print cmd
        return cmd

    def get_fake_output(self):
        fake_output = {0:
            {'Running':'<Data><Job><Job_Id>11924718.gridba3.ba.infn.it</Job_Id><Job_Name>cream_470200607</Job_Name><Job_Owner>biomed009@cream-ce-2.ba.infn.it</Job_Owner><resources_used><cput>00:13:56</cput><mem>600632kb</mem><vmem>2231012kb</vmem><walltime>00:17:13</walltime></resources_used><job_state>R</job_state><queue>long</queue><server>gridba3.ba.infn.it</server><Checkpoint>u</Checkpoint><ctime>1330324413</ctime><Error_Path>cream-ce-2.ba.infn.it:/dev/null</Error_Path><exec_host>pccms122.ba.infn.it/15</exec_host><Hold_Types>n</Hold_Types><Join_Path>n</Join_Path><Keep_Files>n</Keep_Files><Mail_Points>n</Mail_Points><mtime>1330324439</mtime><Output_Path>cream-ce-2.ba.infn.it:/dev/null</Output_Path><Priority>0</Priority><qtime>1330324413</qtime><Rerunable>True</Rerunable><Resource_List><neednodes>lcgpbs</neednodes></Resource_List><session_id>9163</session_id><Shell_Path_List>/bin/bash</Shell_Path_List><stagein>cream_470200607.proxy@cream-ce-2.ba.infn.it:/opt/glite/var/cream_sandbox/biomed/_O_GRID_FR_C_FR_O_ISCPIF_CN_Emmanuel_Faure_biomed_Role_NULL_Capability_NULL_biomed009/proxy/13303239952E319227wms042Eegee2Dsee2Eorg12483597912344,CREAM470200607_jobWrapper.sh@cream-ce-2.ba.infn.it:/opt/glite/var/cream_sandbox/biomed/_O_GRID_FR_C_FR_O_ISCPIF_CN_Emmanuel_Faure_biomed_Role_NULL_Capability_NULL_biomed009/47/CREAM470200607/CREAM470200607_jobWrapper.sh</stagein><stageout>err_cream_470200607_StandardOutput@cream-ce-2.ba.infn.it:/opt/glite/var/cream_sandbox/biomed/_O_GRID_FR_C_FR_O_ISCPIF_CN_Emmanuel_Faure_biomed_Role_NULL_Capability_NULL_biomed009/47/CREAM470200607/StandardError,out_cream_470200607_StandardOutput@cream-ce-2.ba.infn.it:/opt/glite/var/cream_sandbox/biomed/_O_GRID_FR_C_FR_O_ISCPIF_CN_Emmanuel_Faure_biomed_Role_NULL_Capability_NULL_biomed009/47/CREAM470200607/StandardOutput</stageout><substate>42</substate><Variable_List>PBS_O_QUEUE=long,PBS_O_HOME=/home/biomed009,PBS_O_LANG=en_US.UTF-8,PBS_O_LOGNAME=biomed009,PBS_O_PATH=/usr/kerberos/bin:/opt/edg/bin:/opt/glite/bin:/opt/globus/bin:/opt/lcg/bin:/usr/bin:/bin:/home/biomed009/bin,PBS_O_MAIL=/var/spool/mail/biomed009,PBS_O_SHELL=/bin/sh,PBS_O_HOST=cream-ce-2.ba.infn.it,PBS_SERVER=gridba3.ba.infn.it,PBS_O_WORKDIR=/opt/glite/var/tmp</Variable_List><euser>biomed009</euser><egroup>biomed</egroup><hashname>11924718.gridba3.ba.infn.it</hashname><queue_rank>9928560</queue_rank><queue_type>E</queue_type><etime>1330324413</etime><submit_args>/tmp/cream_470200607</submit_args><start_time>1330324439</start_time><start_count>1</start_count><fault_tolerant>False</fault_tolerant><submit_host>cream-ce-2.ba.infn.it</submit_host><init_work_dir>/opt/glite/var/tmp</init_work_dir></Job></Data>',
            'Queued':'<Data><Job><Job_Id>11910242.gridba3.ba.infn.it</Job_Id><Job_Name>cream_311650187</Job_Name><Job_Owner>cdf007@cream-ce-2.ba.infn.it</Job_Owner><job_state>Q</job_state><queue>long</queue><server>gridba3.ba.infn.it</server><Checkpoint>u</Checkpoint><ctime>1330212926</ctime><Error_Path>cream-ce-2.ba.infn.it:/dev/null</Error_Path><Hold_Types>n</Hold_Types><Join_Path>n</Join_Path><Keep_Files>n</Keep_Files><Mail_Points>n</Mail_Points><mtime>1330212926</mtime><Output_Path>cream-ce-2.ba.infn.it:/dev/null</Output_Path><Priority>0</Priority><qtime>1330212926</qtime><Rerunable>True</Rerunable><Resource_List><neednodes>lcgpbs</neednodes></Resource_List><Shell_Path_List>/bin/bash</Shell_Path_List><stagein>cream_311650187.proxy@cream-ce-2.ba.infn.it:/opt/glite/var/cream_sandbox/cdf/_DC_gov_DC_fnal_O_Fermilab_OU_Robots_CN_cnafcaf_CN_cdf_CN_Silvia_Amerio_CN_UID_amerio_cdf_Role_NULL_Capability_NULL_cdf007/proxy/1330210677_71168112483597912344,CREAM311650187_jobWrapper.sh@cream-ce-2.ba.infn.it:/opt/glite/var/cream_sandbox/cdf/_DC_gov_DC_fnal_O_Fermilab_OU_Robots_CN_cnafcaf_CN_cdf_CN_Silvia_Amerio_CN_UID_amerio_cdf_Role_NULL_Capability_NULL_cdf007/31/CREAM311650187/CREAM311650187_jobWrapper.sh</stagein><stageout>err_cream_311650187_StandardOutput@cream-ce-2.ba.infn.it:/opt/glite/var/cream_sandbox/cdf/_DC_gov_DC_fnal_O_Fermilab_OU_Robots_CN_cnafcaf_CN_cdf_CN_Silvia_Amerio_CN_UID_amerio_cdf_Role_NULL_Capability_NULL_cdf007/31/CREAM311650187/StandardError,out_cream_311650187_StandardOutput@cream-ce-2.ba.infn.it:/opt/glite/var/cream_sandbox/cdf/_DC_gov_DC_fnal_O_Fermilab_OU_Robots_CN_cnafcaf_CN_cdf_CN_Silvia_Amerio_CN_UID_amerio_cdf_Role_NULL_Capability_NULL_cdf007/31/CREAM311650187/StandardOutput</stageout><substate>11</substate><Variable_List>PBS_O_QUEUE=long,PBS_O_HOME=/home/cdf007,PBS_O_LANG=en_US.UTF-8,PBS_O_LOGNAME=cdf007,PBS_O_PATH=/usr/kerberos/bin:/opt/edg/bin:/opt/glite/bin:/opt/globus/bin:/opt/lcg/bin:/usr/bin:/bin:/home/cdf007/bin,PBS_O_MAIL=/var/spool/mail/cdf007,PBS_O_SHELL=/bin/sh,PBS_O_HOST=cream-ce-2.ba.infn.it,PBS_SERVER=gridba3.ba.infn.it,PBS_O_WORKDIR=/opt/glite/var/tmp</Variable_List><euser>cdf007</euser><egroup>cdf</egroup><queue_rank>9914130</queue_rank><queue_type>E</queue_type><etime>1330212926</etime><submit_args>/tmp/cream_311650187</submit_args><fault_tolerant>False</fault_tolerant><submit_host>cream-ce-2.ba.infn.it</submit_host><init_work_dir>/opt/glite/var/tmp</init_work_dir></Job></Data>'},
            1:{'Unknown':'qstat: Unknown Job Id'}
            }
        return fake_output

class Qsub(command.Command):
    def __init__(self, option, user = ''):
    
        self.__option__ = option
        self.__user__ = user

    def _get_command(self):
        cmd = 'su -l %s -c "qsub %s"' % (self.__user__, self.__option__)
        #print cmd
        return cmd

    def get_fake_output(self):
        fake_output = '11924801.gridba3.ba.infn.it'
        return ['0', fake_output]

class PbsCommands(batch_system.BatchSystem):
    """ Set of lsf batch system commands"""
    def __init__(self, expiration = 15):

        self.expiration = expiration

    def _map_qstat_in_bjobs(self):
        map = {'Job_Id':'JOBID',
            'euser':'USER',
            'job_state':'STAT',
            'queue':'QUEUE',
            'submit_host':'FROM_HOST',
            'exec_host':'EXEC_HOST',
            'Job_Name':'JOB_NAME',
            'start_time':'SUBMIT_TIME'}
        return map

    def badmin(self, option, hostname, fake = False, fake_status = ''):
        """badmin"""

        b_admin = Pbsnodes(option, hostname)
        b_admin_output = b_admin.get_output(fake = fake)

        if fake:
            fake_output = b_admin.get_fake_output()
            for cmd_status in fake_output.keys():
                for node_status in fake_output[cmd_status].keys():
                    if node_status == fake_status:
                        b_admin_output.append(cmd_status)
                        b_admin_output.append(
                            fake_output[cmd_status][node_status]
                        )
                        break

        #print b_admin_output
        return b_admin_output

    def bhost(self, hostname, fake = False, fake_status = 'Unknown'):
        """bhost""" 

        b_host = Pbsnodes('', hostname)
        b_host_output = b_host.get_output(fake = fake)

        if fake:
            fake_output = b_host.get_fake_output()
            for cmd_status in fake_output.keys():
                for node_status in fake_output[cmd_status].keys():
                    if node_status == fake_status:
                        b_host_output.append(cmd_status)
                        b_host_output.append(
                            fake_output[cmd_status][node_status]
                        )
                        break

        if b_host_output[0] == 0:
            try:
                host_info = ElementTree.fromstring(b_host_output[1])

                for i in host_info.getiterator():
                    if i.tag == 'state':
                        host_status = i.text.upper()
            except:
                host_status = 'UNKNOWN'
        else:
            if 'UNKNOWN' in b_host_output[1].upper():
                host_status = 'UNKNOWN'
            else:
                host_status = 'WAITING'

        #print b_host_output
        return b_host_output

    def bjobs(self, jobid = '', user = '', fake = False, fake_status = 'Unknown'):
        """bjobs"""

        qstat_mask = {}
        for x in self._map_qstat_in_bjobs().keys():
            qstat_mask[x] = 'Empty'

        try:
            job_id = jobid.split('.')[0]
        except:
            job_id = jobid

        b_jobs = Qstat(jobid = job_id, user = user)
        b_jobs_output = b_jobs.get_output(fake = fake)

        if fake:
            fake_output = b_jobs.get_fake_output()
            for cmd_status in fake_output.keys():
                for queue_status in fake_output[cmd_status].keys():
                    if queue_status == fake_status:
                        b_jobs_output.append(cmd_status)
                        b_jobs_output.append(
                            fake_output[cmd_status][queue_status]
                        )
                        break

        if b_jobs_output[0] == 0:

            job_info = ElementTree.fromstring(b_jobs_output[1])
            for i in job_info.getiterator():
                for x in qstat_mask.keys():
                    if x == i.tag:
                        if i.tag == 'Job_Id':
                            qstat_mask[x] = i.text.split('.')[0]
                        else:
                            qstat_mask[x] = i.text

            if qstat_mask['job_state'] == 'Q':
                qstat_mask['job_state'] = 'PEND'
            elif qstat_mask['job_state'] == 'R':
                qstat_mask['job_state'] = 'RUN'
            elif 'UNKNOWN' in b_jobs_output[1].upper():
                qstat_mask['job_state'] = 'DONE'
            else:
                qstat_mask['job_state'] = 'WAITING'
        else:
            # handle the case where the job is finished
            # or the batch system does not respond to the query because busy
            if 'UNKNOWN' in b_jobs_output[1].upper():
                qstat_mask['job_state'] = 'DONE'
            else:
                qstat_mask['job_state'] = 'WAITING'
            if job_id != '':
                qstat_mask['Job_Id'] = job_id
            if user != '':
                qstat_mask['euser'] = user

        #fr = 'JOBID USER STAT QUEUE FROM_HOST EXEC_HOST JOB_NAME SUBMIT_TIME\n'
        sr = '%s %s %s %s %s %s %s %s' % (qstat_mask['Job_Id'],
            qstat_mask['euser'], qstat_mask['job_state'],
            qstat_mask['queue'], qstat_mask['submit_host'],
            qstat_mask['exec_host'], qstat_mask['Job_Name'],
            qstat_mask['start_time'])

        #b_jobs_new_output = [b_jobs_output[0], fr+sr]
        b_jobs_new_output = [b_jobs_output[0], sr]
        #print b_jobs_new_output
        return b_jobs_new_output

    def bkill(self, jobid, user = '', fake = False):
        """bkill"""

        b_kill = Qdel(jobid, user = user)
        b_kill_output = b_kill.get_output(fake = fake)

        if fake:
            b_kill_output = b_kill.get_fake_output()

        #print b_kill_output
        return b_kill_output

    def breserve(self, action, id, hostname = '', fake = False):
        """breserve"""

        b_reserve = Qalter(action, id, hostname = hostname)
        b_reserve_output = b_reserve.get_output(fake = fake)

        if fake:
            b_reserve_output = b_reserve.get_fake_output()

        #print b_reserve_output
        return b_reserve_output

    def brun(self, hostname, jobid, fake = False):
        """brun"""

        b_run = Qrun(hostname, jobid)
        b_run_output = b_run.get_output(fake = fake)

        if fake:
            b_run_output = b_run.get_fake_output()

        #print b_run_output
        return b_run_output

    def bsub(self, option, user = '', fake = False):
        """bsub""" 

        b_sub = Qsub(option, user = user)
        b_sub_output = b_sub.get_output(fake = fake)

        if fake:
            b_sub_output = b_sub.get_fake_output()

        #print b_sub_output
        return b_sub_output
